//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Detector/DetectorItem.h
//! @brief     Defines class DetectorItem.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DETECTOR_DETECTORITEM_H
#define BORNAGAIN_GUI_MODEL_DETECTOR_DETECTORITEM_H

#include "GUI/Model/Descriptor/SelectionProperty.h"
#include "GUI/Model/Descriptor/SpanProperty.h"
#include "GUI/Model/Detector/ResolutionFunctionItemCatalog.h"

class Beam;
class IDetector;
class IResolutionFunction2D;
class MaskList;
class ResolutionFunctionItem;

class DetectorItem {
public:
    DetectorItem();
    ~DetectorItem();

    std::unique_ptr<IDetector> createDetector() const;

    void importMasks(const MaskList* item);
    void setMasks(MaskList* item);

    SelectionProperty<ResolutionFunctionItemCatalog>& resolutionFunctionSelection()
    {
        return m_resolutionFunction;
    }

    template <typename T> T* setResolutionFunctionType();

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

    SpanProperty& phiAxis() { return m_phiAxis; }
    const SpanProperty& phiAxis() const { return m_phiAxis; }

    SpanProperty& alphaAxis() { return m_alphaAxis; }
    const SpanProperty& alphaAxis() const { return m_alphaAxis; }

    bool expandResolutionFunc = true;

private:
    std::unique_ptr<IResolutionFunction2D> createResolutionFunction() const;

    //! for creation of domain detector; only filled and relevant in jobs
    std::unique_ptr<MaskList> m_maskContainerItem; // without serializtion
    SelectionProperty<ResolutionFunctionItemCatalog> m_resolutionFunction;

    SpanProperty m_phiAxis;
    SpanProperty m_alphaAxis;
};

template <typename T> T* DetectorItem::setResolutionFunctionType()
{
    m_resolutionFunction.setCurrentItem<T>();
    return dynamic_cast<T*>(m_resolutionFunction.currentItem());
}

#endif // BORNAGAIN_GUI_MODEL_DETECTOR_DETECTORITEM_H
