//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Mask/MaskeditorListmodel.h
//! @brief     Defines class MaskeditorListmodel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MASK_MASKEDITORLISTMODEL_H
#define BORNAGAIN_GUI_MODEL_MASK_MASKEDITORLISTMODEL_H

#include <QAbstractListModel>
#include <QItemSelectionModel>
#include <QModelIndex>
#include <QVariant>

class MaskItem;
class MaskList;
class OverlayItem;
class ProjectionList;
class RegionOfInterestItem;

//! Provides interfaces to a MaskList, allowing its contents to be displayed and modified
//! using the Qt mechanisms.

class MaskeditorListmodel : public QAbstractListModel {
    Q_OBJECT
public:
    MaskeditorListmodel(MaskList* item);
    ~MaskeditorListmodel();

    //... Override functions from QAbstractListModel

    QVariant data(const QModelIndex& index, int role) const override;
    int rowCount(const QModelIndex& parent) const override;

    //... Other functions

    void insertMask(int row, MaskItem* item);

    void removeMask(MaskItem* item);

    //! Removes all masks
    void clear();

    void moveMask(int from_row, int to_row);
    void removeMaskAt(int row);

    QModelIndex indexOfItem(const OverlayItem* item) const; // TODO: change this to MaskItem*
    MaskItem* itemForIndex(const QModelIndex& index) const;

    QItemSelectionModel* maskSelectionModel() { return m_selection_model.get(); }

    MaskList* maskContItem() { return m_mask_container_item; }
    const MaskList* maskContItem() const { return m_mask_container_item; }
    ProjectionList* projnItem();
    const ProjectionList* projnItem() const;

private:
    MaskList* m_mask_container_item;
    std::unique_ptr<QItemSelectionModel> m_selection_model;
};

#endif // BORNAGAIN_GUI_MODEL_MASK_MASKEDITORLISTMODEL_H
