//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Model/DatafilesModel.h
//! @brief     Defines class DatafilesModel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MODEL_DATAFILESMODEL_H
#define BORNAGAIN_GUI_MODEL_MODEL_DATAFILESMODEL_H

#include "GUI/Model/Model/DatafilesCleaner.h"
#include <QObject>
#include <QXmlStreamWriter>

class DataItem;
class DatafileItem;
class InstrumentModel;
class MessageService;

//! The DatafilesModel class is a model to store all imported DatafileItem's.
class DatafilesModel : public QObject {
    Q_OBJECT
public:
    explicit DatafilesModel();
    ~DatafilesModel();

    QVector<DataItem*> dataItems() const;
    void clear();

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

    void writeDatafiles(const QString& projectDir);
    void readDatafiles(const QString& projectDir, MessageService* messageService);

    DatafileItem* insertDataItem(int rank);
    QVector<DatafileItem*> realItems() const;
    QVector<DatafileItem*> realItems(int rank) const;

    QVector<DatafileItem*> realItems1D() const;
    QVector<DatafileItem*> realItems2D() const;

    void removeDatafileItem(DatafileItem* realItem);

    QStringList realItemNames() const;

    int selectedIndex() const { return m_selectedIndex; }
    void setSelectedIndex(int index) { m_selectedIndex = index; }

    int selectedRank() const { return m_selectedRank; }
    void setSelectedRank(int rank) { m_selectedRank = rank; }

    DatafileItem* pushNewItem(); // private except for test

private:
    DatafilesCleaner dataFilesCleaner;
    std::vector<std::unique_ptr<DatafileItem>> m_realItems;
    int m_selectedIndex = -1;
    int m_selectedRank = -1;
};

#endif // BORNAGAIN_GUI_MODEL_MODEL_DATAFILESMODEL_H
