//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Setup/ProjectionsSaver.h
//! @brief     Defines class ProjectionsSaver.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SETUP_PROJECTIONSSAVER_H
#define BORNAGAIN_GUI_VIEW_SETUP_PROJECTIONSSAVER_H

#include "GUI/Model/Mask/ProjectionList.h"
#include <QString>
#include <QVector>
#include <QWidget>
#include <memory>

class Data2DItem;
class Datafield;

//! Assistant class which save all projections of IndensityDataItem into ASCII file.

class ProjectionsSaver {
public:
    ProjectionsSaver();
    ~ProjectionsSaver();

    void saveProjections(QWidget* parent, Data2DItem* data2DItem);

private:
    struct Projection {
        double axis_value; //!< value on axis where projection has been made
        QVector<double> bin_values;
    };

    struct ProjectionsData {
        bool is_horizontal;
        QVector<double> bin_centers;
        QVector<Projection> projections;
    };

    QString projectionsToString(Qt::Orientation projectionsType, Data2DItem* data2DItem);

    const Datafield* m_field;
    ProjectionsData projectionsData(Qt::Orientation projectionsType, Data2DItem* data2DItem);
    QVector<const LineItem*> projectionItems(Qt::Orientation projectionsType,
                                             Data2DItem* data2DItem) const;

    QString projectionFileHeader(ProjectionsData& projectionsData);
};

#endif // BORNAGAIN_GUI_VIEW_SETUP_PROJECTIONSSAVER_H
